/***********************************************************************
 * Copyright 2011  Geoffry Song <goffrie@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License or (at your option) version 3 or any later version
 * accepted by the membership of KDE e.V. (or its successor approved
 * by the membership of KDE e.V.), which shall act as a proxy
 * defined in Section 14 of version 3 of the license.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ***********************************************************************/

#include "playerApplication.h"

#include "mainWindow.h"

namespace Dragon
{

PlayerApplication::PlayerApplication()
    : KUniqueApplication(true, true)
    , m_mainWindow(0)
{
}

PlayerApplication::~PlayerApplication()
{
    if (m_mainWindow) {
        m_mainWindow = 0;
        delete m_mainWindow;
    }
}

int PlayerApplication::newInstance()
{
    if (!m_mainWindow)
        m_mainWindow = new Dragon::MainWindow;

    if (restoringSession())
        m_mainWindow->restore(1, false);
    else
        m_mainWindow->parseArgs();

    m_mainWindow->show();
    return KUniqueApplication::newInstance();
}

} // namespace Dragon
